@@ -52,6 +52,7 @@ public class ApplicationStatus {
 
     private static class ActivityInfo {
         private int mStatus = ActivityState.DESTROYED;
+        private boolean mHasWindowFocus;
         private final ObserverList<ActivityStateListener> mListeners = new ObserverList<>();
 
         /**
@@ -69,6 +70,20 @@ public class ApplicationStatus {
             mStatus = status;
         }
 
+        /**
+         * @return Whether the activity has window focus.
+         */
+        public boolean hasWindowFocus() {
+            return mHasWindowFocus;
+        }
+
+        /**
+         * @param hasWindowFocus Whether the activity has window focus.
+         */
+        public void setHasWindowFocus(boolean hasWindowFocus) {
+            mHasWindowFocus = hasWindowFocus;
+        }
+
         /**
          * @return A list of {@link ActivityStateListener}s listening to this activity.
          */
@@ -77,9 +92,14 @@ public class ApplicationStatus {
         }
     }
 
-    /** A map of which observers listen to state changes from which {@link Activity}. */
+    /**
+     * A map of which observers listen to state changes from which {@link Activity}.
+     *
+     * <p>Access to the cached state should be synchronized by this map.
+     **/
+    @GuardedBy("sActivityInfo")
     private static final Map<Activity, ActivityInfo> sActivityInfo =
-            Collections.synchronizedMap(new HashMap<Activity, ActivityInfo>());
+            new HashMap<Activity, ActivityInfo>();
 
     /** A map to cache TaskId for each {@link Activity}. */
     public static final Map<Activity, Integer> sActivityTaskId =
@@ -279,6 +299,12 @@ public class ApplicationStatus {
 
         public void onWindowFocusChanged(boolean hasFocus) {
             mCallback.onWindowFocusChanged(hasFocus);
+            synchronized (sActivityInfo) {
+                ActivityInfo info = sActivityInfo.get(mActivity);
+                if (info != null) {
+                    info.setHasWindowFocus(hasFocus);
+                }
+            }
 
             if (sWindowFocusListeners != null) {
                 for (WindowFocusChangedListener listener : sWindowFocusListeners) {
@@ -534,8 +560,10 @@ public class ApplicationStatus {
     public static int getStateForActivity(@Nullable Activity activity) {
         assert isInitialized();
         if (activity == null) return ActivityState.DESTROYED;
-        ActivityInfo info = sActivityInfo.get(activity);
-        return info != null ? info.getStatus() : ActivityState.DESTROYED;
+        synchronized (sActivityInfo) {
+            ActivityInfo info = sActivityInfo.get(activity);
+            return info != null ? info.getStatus() : ActivityState.DESTROYED;
+        }
     }
 
     /**
@@ -574,7 +602,9 @@ public class ApplicationStatus {
     @AnyThread
     public static boolean isEveryActivityDestroyed() {
         assert isInitialized();
-        return sActivityInfo.isEmpty();
+        synchronized (sActivityInfo) {
+            return sActivityInfo.isEmpty();
+        }
     }
 
     /**
@@ -587,11 +617,13 @@ public class ApplicationStatus {
     @AnyThread
     public static boolean isTaskVisible(int taskId) {
         assert isInitialized();
-        for (Map.Entry<Activity, ActivityInfo> entry : sActivityInfo.entrySet()) {
-            if (getTaskId(entry.getKey()) == taskId) {
-                @ActivityState int state = entry.getValue().getStatus();
-                if (state == ActivityState.RESUMED || state == ActivityState.PAUSED) {
-                    return true;
+        synchronized (sActivityInfo) {
+            for (Map.Entry<Activity, ActivityInfo> entry : sActivityInfo.entrySet()) {
+                if (getTaskId(entry.getKey()) == taskId) {
+                    @ActivityState int state = entry.getValue().getStatus();
+                    if (state == ActivityState.RESUMED || state == ActivityState.PAUSED) {
+                        return true;
+                    }
                 }
             }
         }
@@ -617,6 +649,24 @@ public class ApplicationStatus {
         return !inaccessibleActivities.isEmpty();
     }
 
+    /**
+     * Returns true if there is any activity with window focus.
+     *
+     * @return Whether any Activity under this Application has window focus.
+     */
+    public static boolean hasWindowFocusedActivity() {
+        assert isInitialized();
+        synchronized (sActivityInfo) {
+            for (Map.Entry<Activity, ActivityInfo> entry : sActivityInfo.entrySet()) {
+                // Check cached value in ActivityInfo.
+                if (entry.getValue().hasWindowFocus()) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
     /**
      * Registers the given listener to receive state changes for all activities.
      *
@@ -647,13 +697,15 @@ public class ApplicationStatus {
         assert isInitialized();
         assert activity != null;
 
-        ActivityInfo info = sActivityInfo.get(activity);
-        assert info != null
-                : String.format(
-                        "Found untracked Activity: %s isDestroyed=%s isFinishing=%s",
-                        activity, activity.isDestroyed(), activity.isFinishing());
-        assert info.getStatus() != ActivityState.DESTROYED : activity.toString();
-        info.getListeners().addObserver(listener);
+        synchronized (sActivityInfo) {
+            ActivityInfo info = sActivityInfo.get(activity);
+            assert info != null
+                    : String.format(
+                            "Found untracked Activity: %s isDestroyed=%s isFinishing=%s",
+                            activity, activity.isDestroyed(), activity.isFinishing());
+            assert info.getStatus() != ActivityState.DESTROYED : activity.toString();
+            info.getListeners().addObserver(listener);
+        }
     }
 
     /**
@@ -800,6 +852,11 @@ public class ApplicationStatus {
         return sApplicationStateListeners == null ? 0 : sApplicationStateListeners.size();
     }
 
+    public static @Nullable ObserverList<TaskVisibilityListener>
+            getTaskVisibilityListenersForTesting() {
+        return sTaskVisibilityListeners;
+    }
+
     @NativeMethods
     interface Natives {
         // Called to notify the native side of state changes.
